function on.paint(gc)
    if cursor then
        drawgrid(gc,drawscale)
    end
    for k,v in pairs(fills) do
        if v[3] then
            gc:setPen("thin","dashed")
        else
            gc:setPen("thin","smooth")
        end
        drawline(gc,drawscale,v[1],v[2])
    end
    if blink and cursor then
        gc:setPen("thin","dashed")
        gc:setColorRGB(255,0,0)
        drawline(gc,drawscale,x,y)
    end
end

x=1
y=1
divided = true
drawscale = 20

fills = {}

blink = true
cursor = false

dashed = false



function drawgrid(gc,scale)
    width = platform.window:width()
    height = platform.window:height()
    up = false
    for i=5,width,scale do
        if up then
            up = false
            for j=5+(scale/2),height,scale do
                gc:drawLine(i,j,i,j)
            end
        else
            up = true
            for j=5,height,scale do
                gc:drawLine(i,j,i,j)
            end
        end
    end
end

function drawline(gc,scale,x,y)
    width = platform.window:width()
    height = platform.window:height()
    if istype(x,1) then
        gc:drawLine(5+((x-1)/2*scale),5+((y-1)*scale),5+((x-1)/2*scale),5+(y*scale))
    end
    if istype(x,2) then
        gc:drawLine(5+((math.ceil(x/2)-1)*scale),5+((math.floor(y/2))*scale),5+((math.ceil(x/2))*scale),(5-(scale/2))+(math.ceil(y/2)*scale))
    end
    if istype(x,3) then
       gc:drawLine(5+((x-1)*scale)/2,5+(scale/2)+((y-1)*scale),5+((x-1)*scale)/2,5+(scale/2)+(y*scale))
    end
    if istype(x,4) then
        gc:drawLine(5+(x/2*scale),5+((math.floor(y/2))*scale),5+(((x/2)-1)*scale),(5-(scale/2))+(math.ceil(y/2)*scale))
    end
end

function round(num,idp)
    return tonumber(string.format("%." .. (idp or 0) .. "f", num))
end

function isint(x)
    return x==round(x)
end

function istype(x,r)
    return isint((x-r)/4)
end




function on.arrowLeft()
    x = x-1
    if divided then
        y=math.floor(y*2)
    else
        y=math.floor(y/2)
    end
    divided = not divided
    platform.window:invalidate()
    resetTimer()
    fixCoords()
end

function on.arrowRight()
    x = x+1
    if divided then
        y=math.floor(y*2)
    else
        y=math.floor(y/2)
    end
    divided = not divided
    platform.window:invalidate()
    resetTimer()
    fixCoords()
end

function on.arrowUp()
    y=y-1
    platform.window:invalidate()
    resetTimer()
    fixCoords()
end

function on.arrowDown()
    y=y+1
    platform.window:invalidate()
    resetTimer()
    fixCoords()
end

function on.charIn(ch)
    if ch == "+" then
        drawscale = drawscale + 1
        document.markChanged()
    end
    if ch == "-" then
        drawscale = drawscale - 1
        document.markChanged()
    end
    if ch == "m" or ch == "M" then
        dashed = not dashed
    end
    platform.window:invalidate()
    resetTimer()
end

function on.enterKey()
    draw = true
    for k,v in pairs(fills) do
        if v[1] == x and v[2] == y then
            draw = false
        end
    end
    if draw then
        table.insert(fills, {x,y,dashed})
        document.markChanged()
    end
    platform.window:invalidate()
    resetTimer()
end

function on.backspaceKey()
    draw = nil
    for k,v in pairs(fills) do
        if v[1] == x and v[2] == y then
            draw = k
        end
    end
    if draw then
        table.remove(fills, draw)
        document.markChanged()
    end
    platform.window:invalidate()
    resetTimer()
end

function on.timer()
    blink = not blink
    timer.start(.5)
    platform.window:invalidate()
end

function resetTimer()
    blink = true
    timer.start(.5)
end

function fixCoords()
    if x<1 then x=1 end
    if y<1 then y=1 end
end

function on.restore(toload)
    fills = toload[1]
    test = drawscale
    drawscale = toload[2]
end

function on.save()
    return {fills,drawscale}
end

function on.activate()
    cursor = true
    resetTimer()
    platform.window:invalidate()
end

function on.deactivate()
    cursor = false
    platform.window:invalidate()
end